package	jp.co.sds_corp.transfer.output;

import java.net.MalformedURLException;
import java.rmi.Naming;
import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.rmi.server.UnicastRemoteObject;

import jp.co.sds_corp.transfer.IDataKey;
import jp.co.sds_corp.transfer.IReceiveListener;
import jp.co.sds_corp.transfer.ReceiveEvent;
import jp.co.sds_corp.transfer.ReceiverSupport;
import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.log.CmnLogFactory;
import jp.co.sds_corp.util.log.CmnLogger;

/**
* RMIを使用してデータを外部に送信します。
host,port,nameの各要素で、RMIAdapterを登録するRMI registryを指定します。
<PRE>
&lt;Output name="RMI" class="jp.co.sds_corp.transfer.output.RMIAdapter"&gt;
	&lt;host&gt;localhost&lt;/host&gt;
	&lt;port&gt;1091&lt;/port&gt;
	&lt;name&gt;RMIAdapter&lt;/name&gt;
&lt;/Output&gt;
</PRE>
*/
public class RMIAdapter extends AbstractOutput
{
	static final CmnLogger logger = CmnLogFactory.getLogger();
	ReceiverSupport support = new ReceiverSupport();
	String host;
	int port;
	String name;
	String url;
	Receiver rcv;

	/**
	* 指定された初期情報で初期化します。
	* @param	config	初期情報
	*/
	public void init( IConfiguration config ) throws Exception {
		super.init( config );
		rcv = new Receiver();
		host = config.get( "host" );
		port = Integer.valueOf( config.get( "port" ) ).intValue();
		name = config.get( "name" );

		Registry	reg;
		try {
			reg = LocateRegistry.createRegistry( port );
		} catch (RemoteException e) {
			// Registry起動済み
			reg = LocateRegistry.getRegistry( port );
		}
		url = "rmi://" + host + ":" + port + "/" + name;
		Naming.bind( url, rcv );
	}

	/**
	* このサービスを停止します。
	*/
	public void stop() {
		try {
			if ( url != null ) {
				Naming.unbind( url );
			}
		} catch ( Exception e ) {
			logger.error( this, "unbind error", e );
		}
	}

	/**
	* RMI registry のホスト名(host)を取得します。
	* @return	hostの内容
	*/
	public String getHost() {
		return host;
	}
	/**
	* RMI registry のポート番号(port)を取得します。
	* @return	portの内容
	*/
	public int getPort() {
		return port;
	}
	/**
	* RMI registry の登録名(name)を取得します。
	* @return	nameの内容
	*/
	public String getName() {
		return name;
	}
	/**
	* RMI registry のURL(url)を取得します。
	* @return	urlの内容
	*/
	public String getUrl() {
		return url;
	}

	/**
	* データに何らかの変更があったときに呼び出されます。
	* @param	e		イベントソースおよびデータの変更内容を指定します。
	*/
	public void received( ReceiveEvent e ) throws Exception {
		if ( isDebug() ) {
			logger.debug( this, e.toString() );
		}
		rcv.fireReceive( e );
	}

	/**
	* RMI
	*/
	public class Receiver extends UnicastRemoteObject implements IRMIAdapter
	{
		private static final long serialVersionUID = 1L;

		public Receiver() throws RemoteException, MalformedURLException {
		}

		/**
		* データ受信を受け取るオブジェクトを追加します。
		* @param	l		データ受信を受け取るオブジェクト
		*/
		public void addReceiveListener( IReceiveListener l )
			throws RemoteException
		{
			support.addReceiveListener( (IReceiveListener)l );
		}
		/**
		* データ受信を受け取るオブジェクトを削除します。
		* @param	l		データ受信を受け取るオブジェクト
		*/
		public void removeReceiveListener( IReceiveListener l )
			throws RemoteException
		{
			support.removeReceiveListener( l );
		}
		/**
		* データ受信を受け取るオブジェクトを追加します。
		* @param	key		対象のキー
		* @param	l		データ受信を受け取るオブジェクト
		*/
		public void addReceiveListener( IDataKey key, IReceiveListener l )
			throws RemoteException
		{
			support.addReceiveListener( key, l );
		}
		/**
		* データ受信を受け取るオブジェクトを削除します。
		* @param	key		対象のキー
		* @param	l		データ受信を受け取るオブジェクト
		*/
		public void removeReceiveListener( IDataKey key, IReceiveListener l )
			throws RemoteException
		{
			support.removeReceiveListener( key, l );
		}

		/**
		* データ受信イベントを通知します。
		* @param	e		データ受信イベント
		*/
		public void fireReceive( ReceiveEvent e )
			throws Exception
		{
			support.fireReceive( e );
		}
	}

}
